@@ -1,10 +1,5 @@
 package Pod::Weaver::Plugin::Data::Sah::Filter;
 
-our $AUTHORITY = 'cpan:PERLANCAR'; # AUTHORITY
-our $DATE = '2020-02-10'; # DATE
-our $DIST = 'Pod-Weaver-Plugin-Data-Sah-Filter'; # DIST
-our $VERSION = '0.001'; # VERSION
-
 use 5.010001;
 use Moose;
 with 'Pod::Weaver::Role::AddTextToSection';
@@ -13,8 +8,13 @@ with 'Pod::Weaver::Role::Section';
 use Data::Dmp;
 use File::Temp;
 
+our $AUTHORITY = 'cpan:PERLANCAR'; # AUTHORITY
+our $DATE = '2022-07-16'; # DATE
+our $DIST = 'Pod-Weaver-Plugin-Data-Sah-Filter'; # DIST
+our $VERSION = '0.003'; # VERSION
+
 sub _process_filter_module {
-    no strict 'refs';
+    no strict 'refs'; ## no critic: TestingAndDebugging::ProhibitNoStrict
 
     my ($self, $document, $input, $package) = @_;
 
@@ -34,7 +34,7 @@ sub _process_filter_module {
         require $package_pm;
 
         {
-            no strict 'refs';
+            no strict 'refs'; ## no critic: TestingAndDebugging::ProhibitNoStrict
             $meta = $package->meta;
         }
         $package =~ /\AData::Sah::Filter::\w+::(\w+)::(\w+)\z/
@@ -46,8 +46,54 @@ sub _process_filter_module {
     # add Synopsis section
     {
         my @pod;
-        push @pod, "Use in Sah schema's C<prefilters> (or C<postfilters>) clause:\n\n",
-            " ", dmp([$meta->{target_type} // "str", "prefilters" => ["$rule_cat\::$rule_desc"]]), "\n\n";
+        my $type = $meta->{target_type} // "str";
+        my $filter = ["$rule_cat\::$rule_desc"];
+        my $schema = [$type, "prefilters" => [$filter]];
+
+        push @pod, "=head2 Using in Sah schema's C<prefilters> (or C<postfilters>) clause\n\n",
+            " ", dmp($schema), "\n\n";
+
+        push @pod, "=head2 Using with L<Data::Sah>:\n\n",
+            " use Data::Sah qw(gen_validator);\n",
+            " \n",
+            " my \$schema = ", dmp($schema), ";\n",
+            " my \$validator = gen_validator(\$schema);\n",
+            " if (\$validator->(\$some_data)) { print 'Valid!' }\n\n";
+
+        push @pod, "=head2 Using with L<Data::Sah:Filter> directly:\n\n",
+            " use Data::Sah::Filter qw(gen_filter);\n\n",
+            " my \$filter = gen_filter([", dmp($filter), "]);\n";
+        if ($meta->{might_fail}) {
+            push @pod,
+                " # \$errmsg will be empty/undef when filtering succeeds\n",
+                " my (\$errmsg, \$filtered_value) = \$filter->(\$some_data);\n\n";
+        } else {
+            push @pod, " my \$filtered_value = \$filter->(\$some_data);\n\n";
+        }
+
+        if ($meta->{examples} && @{ $meta->{examples} }) {
+            require Data::Sah::Filter;
+            require Data::Cmp;
+            push @pod, "=head2 Sample data and filtering results\n\n";
+            for my $eg (@{ $meta->{examples} }) {
+                my $filter_rule = ["$rule_cat\::$rule_desc", $eg->{filter_args} // {}];
+                my $filter_code = Data::Sah::Filter::gen_filter(filter_names=>[$filter_rule]);
+                my ($actual_errmsg, $actual_filtered_value);
+                if ($meta->{might_fail}) {
+                    ($actual_errmsg, $actual_filtered_value) = $filter_code->($eg->{value});
+                } else {
+                    $actual_filtered_value = $filter_code->($eg->{value});
+                    $actual_errmsg = undef;
+                }
+                my $correct_filtered_value = exists($eg->{filtered_value}) ?
+                    $eg->{filtered_value} : $eg->{value};
+                push @pod, " ", dmp($eg->{value}), " #",
+                    ($eg->{filter_args} ? " filtered with args ".dmp($eg->{filter_args}).", " : " "),
+                    ($actual_errmsg ? "INVALID ($actual_errmsg)" : "valid"), ", ",
+                    (Data::Cmp::cmp_data($eg->{value}, $actual_filtered_value) == 0 ? "unchanged" : "becomes ".dmp($actual_filtered_value)), "\n";
+            }
+            push @pod, "\n";
+        }
 
         $self->add_text_to_section(
             $document, join("", @pod), 'SYNOPSIS',
@@ -58,6 +104,23 @@ sub _process_filter_module {
             });
     }
 
+    # add Description section
+    {
+        last unless $meta->{description};
+
+        require Markdown::To::POD;
+        my @pod;
+        push @pod, Markdown::To::POD::markdown_to_pod($meta->{description}), "\n\n";
+
+        $self->add_text_to_section(
+            $document, join("", @pod), 'DESCRIPTION',
+            {
+                after_section => ['VERSION', 'NAME', 'SYNOPSIS'],
+                ignore => 1,
+            });
+    }
+
+    #
     $self->log(["Generated POD for '%s'", $filename]);
 }
 
@@ -75,7 +138,7 @@ sub _list_my_filter_modules {
 }
 
 sub _process_filterbundle_module {
-    no strict 'refs';
+    no strict 'refs'; ## no critic: TestingAndDebugging::ProhibitNoStrict
 
     my ($self, $document, $input, $package) = @_;
 
@@ -153,7 +216,7 @@ Pod::Weaver::Plugin::Data::Sah::Filter - Plugin to use when building Data::Sah::
 
 =head1 VERSION
 
-This document describes version 0.001 of Pod::Weaver::Plugin::Data::Sah::Filter (from Perl distribution Pod-Weaver-Plugin-Data-Sah-Filter), released on 2020-02-10.
+This document describes version 0.003 of Pod::Weaver::Plugin::Data::Sah::Filter (from Perl distribution Pod-Weaver-Plugin-Data-Sah-Filter), released on 2022-07-16.
 
 =head1 SYNOPSIS
 
@@ -194,14 +257,6 @@ Please visit the project's homepage at L<https://metacpan.org/release/Pod-Weaver
 
 Source repository is at L<https://github.com/perlancar/perl-Pod-Weaver-Plugin-Data-Sah-Filter>.
 
-=head1 BUGS
-
-Please report any bugs or feature requests on the bugtracker website L<https://rt.cpan.org/Public/Dist/Display.html?Name=Pod-Weaver-Plugin-Data-Sah-Filter>
-
-When submitting a bug or request, please include a test-file or a
-patch to an existing test-file that illustrates the bug or desired
-feature.
-
 =head1 SEE ALSO
 
 L<Data::Sah::Filter>
@@ -214,11 +269,36 @@ L<Sah>
 
 perlancar <perlancar@cpan.org>
 
+=head1 CONTRIBUTING
+
+
+To contribute, you can send patches by email/via RT, or send pull requests on
+GitHub.
+
+Most of the time, you don't need to build the distribution yourself. You can
+simply modify the code, then test via:
+
+ % prove -l
+
+If you want to build the distribution (e.g. to try to install it locally on your
+system), you can install L<Dist::Zilla>,
+L<Dist::Zilla::PluginBundle::Author::PERLANCAR>, and sometimes one or two other
+Dist::Zilla plugin and/or Pod::Weaver::Plugin. Any additional steps required
+beyond that are considered a bug and can be reported to me.
+
 =head1 COPYRIGHT AND LICENSE
 
-This software is copyright (c) 2020 by perlancar@cpan.org.
+This software is copyright (c) 2022, 2020 by perlancar <perlancar@cpan.org>.
 
 This is free software; you can redistribute it and/or modify it under
 the same terms as the Perl 5 programming language system itself.
 
+=head1 BUGS
+
+Please report any bugs or feature requests on the bugtracker website L<https://rt.cpan.org/Public/Dist/Display.html?Name=Pod-Weaver-Plugin-Data-Sah-Filter>
+
+When submitting a bug or request, please include a test-file or a
+patch to an existing test-file that illustrates the bug or desired
+feature.
+
 =cut
